using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using static CommunicationEvents;

/// <summary>
/// a Gadget that checks whether two given circles have equal size and if yes it returns an EqualCirclesFact
/// </summary>
public class EqualCircleGadget : Gadget
{
    /// \copydoc Gadget.s_type
    [Newtonsoft.Json.JsonProperty]
    protected static new string s_type = "EqualCircles";



    public override void _Hit(RaycastHit[] hit)
    {
        if (hit[0].transform.gameObject.layer != LayerMask.NameToLayer("Circle"))
            //No Circles were hit
            ResetGadget();

        string tempFactId = hit[0].transform.GetComponent<FactObject>().URI;
        if (!Workflow.Contains(tempFactId)) // filters double hits / exact ones
            Workflow.Add(tempFactId);

        switch (Workflow.Count)
        {
            case 1: //If no circle was selected before
                break;

            case 2: //If the first circle got already selected

                //Debug.Log("hit it");
                //Debug.Log("data: radius dif" + Mathf.Abs(this.FirstCircle.radius - tempFact.radius) +" ids: 1. "+ this.FirstCircle.Id+", 2."+ tempFact.Id);

                CircleFact tempFact = (CircleFact)StageStatic.stage.factState[Workflow[0]];
                CircleFact FirstCircle = (CircleFact)StageStatic.stage.factState[tempFactId];

                if (Mathf.Approximately(FirstCircle.radius, tempFact.radius))
                    FactManager.AddEqualCirclesFact  (Workflow[0], tempFactId);

                else
                    FactManager.AddUnEqualCirclesFact(Workflow[0], tempFactId);

                ResetGadget();
                break;
        }
    }
}
